#ifndef __STUB__SIGCONTEXT_H__
#define __STUB__SIGCONTEXT_H__

#ifdef __arch64__
#include <asm-sparc64/sigcontext.h>
#else
#include <asm-sparc/sigcontext.h>
#endif

#endif /* __STUB__SIGCONTEXT_H__ */
