/*
 * include/asm-xtensa/serial.h
 *
 * Configuration details for 8250, 16450, 16550, etc. serial ports
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2001 - 2005 Tensilica Inc.
 */

#ifndef _XTENSA_SERIAL_H
#define _XTENSA_SERIAL_H

#include <asm/platform/serial.h>

#endif	/* _XTENSA_SERIAL_H */
