#ifndef __STUB__OPENPROMIO_H__
#define __STUB__OPENPROMIO_H__

#ifdef __arch64__
#include <asm-sparc64/openpromio.h>
#else
#include <asm-sparc/openpromio.h>
#endif

#endif /* __STUB__OPENPROMIO_H__ */
