#ifndef __STUB__MMU_CONTEXT_H__
#define __STUB__MMU_CONTEXT_H__

#ifdef __arch64__
#include <asm-sparc64/mmu_context.h>
#else
#include <asm-sparc/mmu_context.h>
#endif

#endif /* __STUB__MMU_CONTEXT_H__ */
