/*
 *	inftl.h -- defines to support the Inverse NAND Flash Translation Layer
 *
 *	(C) Copyright 2002, Greg Ungerer (gerg@snapgear.com)
 *
 *	$Id: inftl.h,v 1.7 2005/06/13 13:08:45 sean Exp $
 */

#ifndef __MTD_INFTL_H__
#define __MTD_INFTL_H__

#error This is a kernel header. Perhaps include nftl-user.h instead?

#include <linux/mtd/blktrans.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/nftl.h>

#include <mtd/inftl-user.h>

#ifndef INFTL_MAJOR
#define INFTL_MAJOR 96
#endif
#define INFTL_PARTN_BITS 4

#endif /* __MTD_INFTL_H__ */
