#ifndef __STUB__CACHEFLUSH_H__
#define __STUB__CACHEFLUSH_H__

#ifdef __arch64__
#include <asm-sparc64/cacheflush.h>
#else
#include <asm-sparc/cacheflush.h>
#endif

#endif /* __STUB__CACHEFLUSH_H__ */
