#ifndef __STUB__SCATTERLIST_H__
#define __STUB__SCATTERLIST_H__

#ifdef __arch64__
#include <asm-sparc64/scatterlist.h>
#else
#include <asm-sparc/scatterlist.h>
#endif

#endif /* __STUB__SCATTERLIST_H__ */
