#ifndef __STUB__POSIX_TYPES_H__
#define __STUB__POSIX_TYPES_H__

#ifdef __arch64__
#include <asm-sparc64/posix_types.h>
#else
#include <asm-sparc/posix_types.h>
#endif

#endif /* __STUB__POSIX_TYPES_H__ */
