#ifndef __ASM_SH64_HARDWARE_H
#define __ASM_SH64_HARDWARE_H

/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * include/asm-sh64/hardware.h
 *
 * Copyright (C) 2002 Stuart Menefy
 * Copyright (C) 2003 Paul Mundt
 *
 * Defitions of the locations of registers in the physical address space.
 */

#define	PHYS_PERIPHERAL_BLOCK	0x09000000
#define PHYS_DMAC_BLOCK		0x0e000000
#define PHYS_PCI_BLOCK		0x60000000
#define PHYS_EMI_BLOCK		0xff000000

#endif /* __ASM_SH64_HARDWARE_H */
