#ifndef __STUB__KMAP_TYPES_H__
#define __STUB__KMAP_TYPES_H__

#ifdef __arch64__
#include <asm-sparc64/kmap_types.h>
#else
#include <asm-sparc/kmap_types.h>
#endif

#endif /* __STUB__KMAP_TYPES_H__ */
