#ifndef _ASMARM_PAGE_H
#define _ASMARM_PAGE_H

#include <unistd.h>
#define EXEC_PAGESIZE	32768

#define PAGE_SIZE	sysconf (_SC_PAGE_SIZE)
#define PAGE_SHIFT	(getpageshift())
#define PAGE_MASK	(~(PAGE_SIZE-1))

#define PAGE_ALIGN(addr)	(((addr)+PAGE_SIZE-1)&PAGE_MASK)
#endif /* (_ASMARM_PAGE_H) */
