#ifndef _CRIS_CURRENT_H
#define _CRIS_CURRENT_H

struct task_struct;

static __inline__ struct task_struct * get_current(void)
{
        return current_thread_info()->task;
}
 
#define current get_current()

#endif /* !(_CRIS_CURRENT_H) */
