#ifndef __STUB__POSIX_TYPES_H__
#define __STUB__POSIX_TYPES_H__

#ifdef __x86_64__
#include <asm-x86_64/posix_types.h>
#else
#include <asm-i386/posix_types.h>
#endif

#endif /* __STUB__POSIX_TYPES_H__ */
