#ifndef _SPARC64_PAGE_H
#define _SPARC64_PAGE_H

#include <unistd.h>

#define PAGE_SIZE	sysconf (_SC_PAGE_SIZE)
#define PAGE_SHIFT	(getpageshift())
#define PAGE_MASK	(~(PAGE_SIZE-1))

#endif /* (_SPARC64_PAGE_H) */
