#ifndef _LINUX_VERSION_H
#define _LINUX_VERSION_H

#define UTS_RELEASE "2.6.17"
#define LINUX_VERSION_CODE 132625
#define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))

#endif /* _LINUX_VERSION_H */
