#ifndef _ASM_PARAM_H
#define _ASM_PARAM_H

#ifndef HZ
#define HZ 100
#endif

#define EXEC_PAGESIZE	16384

#ifndef NOGROUP
#define NOGROUP		(-1)
#endif

#define MAXHOSTNAMELEN		64	/* max length of hostname */
#define COMMAND_LINE_SIZE	512

#endif /* _ASM_PARAM_H */
