#ifndef _ASM_M32R_SEGMENT_H
#define _ASM_M32R_SEGMENT_H

/* $Id$ */

/* orig : i386 (2.4.18) */

#define __KERNEL_CS	0x10
#define __KERNEL_DS	0x18

#define __USER_CS	0x23
#define __USER_DS	0x2B

#endif  /* _ASM_M32R_SEGMENT_H */
