/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2003 by Ralf Baechle
 */
#ifndef __ASM_MACH_JAZZ_TIMEX_H
#define __ASM_MACH_JAZZ_TIMEX_H

/*
 * Jazz is still using the R4030 100Hz counter
 */
#define CLOCK_TICK_RATE		100

#endif /* __ASM_MACH_JAZZ_TIMEX_H */
