/* $Id: string.h,v 1.36 2001/12/21 00:54:31 davem Exp $
 * string.h: External definitions for optimized assembly string
 *           routines for the Linux Kernel.
 *
 * Copyright (C) 1995,1996 David S. Miller (davem@caip.rutgers.edu)
 * Copyright (C) 1996,1997 Jakub Jelinek (jj@sunsite.mff.cuni.cz)
 */

#ifndef __SPARC_STRING_H__
#define __SPARC_STRING_H__

/* Really, userland/ksyms should not see any of this stuff. */

#endif /* !(__SPARC_STRING_H__) */
