#ifndef _ASM_ATOMIC_H
#define _ASM_ATOMIC_H

typedef struct { volatile int counter; } atomic_t;

#ifdef __mips64
typedef struct { volatile long counter; } atomic64_t;
#endif

#endif /* _ASM_ATOMIC_H */
