#ifndef _ASM_M32R_PARAM_H
#define _ASM_M32R_PARAM_H

/* $Id$ */

/* orig : i386 2.5.67 */

#ifndef HZ
#define HZ 100
#endif

#define EXEC_PAGESIZE	4096

#ifndef NOGROUP
#define NOGROUP		(-1)
#endif

#define MAXHOSTNAMELEN	64	/* max length of hostname */

#endif /* _ASM_M32R_PARAM_H */

