#ifndef __V850_PAGE_H__
#define __V850_PAGE_H__

#include <unistd.h>

#define PAGE_SIZE	sysconf (_SC_PAGE_SIZE)
#define PAGE_SHIFT	(getpageshift())
#define PAGE_MASK	(~(PAGE_SIZE-1))

#ifndef PAGE_OFFSET
#define PAGE_OFFSET	0x0000000
#endif

#endif /* (__V850_PAGE_H__) */
