/*
 * include/asm-v850/byteorder.h -- Endian id and conversion ops
 *
 *  Copyright (C) 2001  NEC Corporation
 *  Copyright (C) 2001  Miles Bader <miles@gnu.org>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file COPYING in the main directory of this
 * archive for more details.
 *
 * Written by Miles Bader <miles@gnu.org>
 */

#ifndef __V850_BYTEORDER_H__
#define __V850_BYTEORDER_H__

#include <asm/types.h>

#include <linux/byteorder/little_endian.h>

#endif /* __V850_BYTEORDER_H__ */
