/*
 *  linux/include/asm-arm/arch-aaed2000/system.h
 *
 *  Copyright (c) 2005 Nicolas Bellido Y Ortega
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */

#ifndef __ASM_ARCH_SYSTEM_H
#define __ASM_ARCH_SYSTEM_H

static __inline__ void arch_idle(void)
{
	cpu_do_idle();
}

static __inline__ void arch_reset(char mode)
{
	cpu_reset(0);
}

#endif /* __ASM_ARCH_SYSTEM_H */
