/*
 *	include/asm-mips/dec/system.h
 *
 *	Generic DECstation/DECsystem bits.
 *
 *	Copyright (C) 2005  Maciej W. Rozycki
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 */
#ifndef __ASM_DEC_SYSTEM_H
#define __ASM_DEC_SYSTEM_H

extern unsigned long dec_kn_slot_base, dec_kn_slot_size;

#endif /* __ASM_DEC_SYSTEM_H */
