#ifndef __STUB__MMU_CONTEXT_H__
#define __STUB__MMU_CONTEXT_H__

#ifdef __x86_64__
#include <asm-x86_64/mmu_context.h>
#else
#include <asm-i386/mmu_context.h>
#endif

#endif /* __STUB__MMU_CONTEXT_H__ */
