#ifndef _ASMCRIS_PARAM_H
#define _ASMCRIS_PARAM_H

/* Currently we assume that HZ=100 is good for CRIS. */

#ifndef HZ
#define HZ 100
#endif

#define EXEC_PAGESIZE	8192

#ifndef NOGROUP
#define NOGROUP		(-1)
#endif

#define MAXHOSTNAMELEN	64	/* max length of hostname */

#endif
